@@ -10,12 +10,20 @@ from ansi2html import Ansi2HTMLConverter
 
 from pytest_tui import __version__
 
+# Next 3 lines are saved from earlier CLI-Config work
+# CLI CONFIG SAVED WORK
 # from pytest_tui.__main__ import Cli
 # from pytest_tui.utils import CONFIGFILE, HTML_OUTPUT_FILE, TERMINAL_OUTPUT_FILE, Results
+
 from pytest_tui.utils import (
-    PYTEST_TUI_FILES_DIR,
     HTML_OUTPUT_FILE,
+    LOG_LEVEL_MAP,
+    PYTEST_TUI_FILES_DIR,
     TERMINAL_OUTPUT_FILE,
+    TUI_FOLD_CONTENT_BEGIN,
+    TUI_FOLD_CONTENT_END,
+    TUI_FOLD_TITLE_BEGIN,
+    TUI_FOLD_TITLE_END,
     Results,
     test_session_starts_results_grabber,
 )
@@ -23,8 +31,8 @@ from pytest_tui.utils import (
 CSS_FILE = Path(__file__).parent / "resources" / "styles.css"
 JS_FILE = Path(__file__).parent / "resources" / "scripts.js"
 
-TAB_METADATA = ["About"]
-TAB_METADATA_COLOR = {"About": "rgba(0, 0, 255, 0.33)"}
+TAB_ABOUT = ["About"]
+TAB_ABOUT_COLOR = {"About": "hsl(199, 100%, 50%)"}
 TABS_RESULTS = [
     "All Tests",
     "Failures",
@@ -35,14 +43,20 @@ TABS_RESULTS = [
     "Reruns",
 ]
 TABS_RESULTS_COLORS = {
-    "All Tests": ("rgba(111, 104, 105, 0.34)", "rgba(128,128,0,0.5)"),
-    "Failures": ("rgba(255, 10, 10, 0.75)", "rgba(255,0,0,0.5)"),
+    "All Tests": ("hsl(64, 11%, 68%)", "rgba(128,128,0,0.5)"),
+    "Failures": ("hsl(348, 78%, 53%)", "rgba(255,0,0,0.5)"),
     "Passes": ("rgba(66, 228, 47, 1)", "rgba(0,0,0,0.5)"),
-    "Skipped": ("rgba(111, 104, 105, 0.34)", "rgba(128,128,0,0.5)"),
-    "Xfails": ("rgba(255, 176, 176, 0.90)", "rgba(0,0,0,0.5)"),
-    "Xpasses": ("rgba(0, 255, 0, 0.43)", "rgba(0,0,0,0.5)"),
+    "Skipped": ("hsl(172, 21%, 59%)", "rgba(128,128,0,0.5)"),
+    "Xfails": ("rgba(255, 176, 176)", "rgba(0,0,0,0.5)"),
+    "Xpasses": ("hsl(141, 45%, 48%)", "rgba(0,0,0,0.5)"),
     "Reruns": ("rgba(220,165.0,1)", "rgba(255,194,0,0.5)"),
 }
+
+TAB_FULL_OUTPUT = ["Full Output"]
+TAB_FULL_OUTPUT_COLOR = {"Full Output": "hsl(195, 100%, 47%)"}
+
+TAB_FOLDED_OUTPUT = ["Folded Output"]
+TAB_FOLDED_OUTPUT_COLOR = {"Folded Output": "rgba(0, 225, 128, 1)"}
 TABS_SECTIONS = [
     "summary_section",
     "failures_section",
@@ -59,8 +73,17 @@ TABS_SECTIONS_COLORS = {
     "errors_section": "#ffc09f",
     "reruns_section": "#f6e3da",
 }
-TAB_FULL_OUTPUT = ["Console Output"]
-TAB_FULL_OUTPUT_COLOR = {"Console Output": "rgba(0, 0, 255, 0.33)"}
+
+TAB_ACTIONS = ["Actions"]
+TAB_ACTIONS_COLOR = {"Actions": "rgba(249, 123, 64, 0.95)"}
+TABS_ACTIONS = {
+    "fold_action": "toggleAllDetails",
+    "hide_action": "toggleDetailsElements",
+}
+TABS_ACTIONS_COLORS = {
+    "fold_action": "#ffee93",
+    "hide_action": "#ffc09f",
+}
 
 
 class TabContent:
@@ -132,7 +155,6 @@ class HtmlPage:
             TABS_RESULTS.remove("Xpasses")
         if not self.results.tui_test_results.all_xfails():
             TABS_RESULTS.remove("Xfails")
-        # if len(self.results.tui_rerun_test_groups) == 0:
         if not self.results.tui_test_results.all_reruns():
             TABS_RESULTS.remove("Reruns")
 
@@ -149,19 +171,16 @@ class HtmlPage:
             + """</b></pre>"""
         )
 
-    def create_test_execution_info(self) -> str:
-        now = (
-            datetime.now(timezone.utc)
-            .replace(microsecond=0)
-            .strftime("%Y-%m-%d %H:%M:%S")
-        )
+    def create_final_test_summary(self) -> str:
         return (
-            """<button class="accordion">Test Execution Info</button><div class="panel"><p><pre>"""
-            + f"""<p>Test run started: {self.results.tui_session_start_time} UTC</p>"""
-            + f"""<p>Test run completed: {self.results.tui_session_end_time} UTC</p>"""
-            + f"""<p>Test run duration: {self.results.tui_session_duration}</p>"""
-            + f"""<p>This report generated: {now} UTC</p>"""
-            + f"""<p>pytest-tui version: {__version__}</p>"""
+            """<button class="accordion">Final Test Summary  (click to expand)</button><div class="panel-closed"><p><pre>"""
+            + self.converter.convert(
+                self.results.tui_sections.short_test_summary.content, full=False
+            )
+            + "\n"
+            + self.converter.convert(
+                self.results.tui_sections.lastline.content, full=False
+            )
             + """</pre></p></div>"""
         )
 
@@ -174,7 +193,7 @@ class HtmlPage:
         )
         ripped = search.groups()[0].encode().decode("unicode-escape") if search else ""
         return (
-            """<button class="accordion">Live Test Session Summary</button><div class="panel"><p><pre>"""
+            """<button class="accordion">Live Test Session Summary  (click to expand)</button><div class="panel-closed"><p><pre>"""
             + self.converter.convert(
                 self.results.tui_sections.lastline.content, full=False
             )
@@ -185,69 +204,138 @@ class HtmlPage:
             + """</pre></p></div>"""
         )
 
-    def create_testrun_summary(self) -> str:
+    def create_test_execution_info(self) -> str:
+        now = (
+            datetime.now(timezone.utc)
+            .replace(microsecond=0)
+            .strftime("%Y-%m-%d %H:%M:%S")
+        )
+        table_attributes = {
+            "id": "test_execution_info",
+            "font-family": "Helvetica, Arial, sans-serif",
+            "font-size": "12px",
+            "border": "ridge",
+            "style": "width:auto%; table-layout: auto;",
+            "border-collapse": "collapse",
+            "text-align": "left",
+            "tr": "nth-child(even) {background-color: #f2f2f2;}",
+        }
+        d = {
+            "Test run started": f"{self.results.tui_session_start_time} UTC",
+            "Test run completed": f"{self.results.tui_session_end_time} UTC",
+            "Test run duration": f"{self.results.tui_session_duration}",
+            "This report generated": f"{now} UTC",
+            "pytest-tui version": f"{__version__}",
+        }
+        dj = json2table.convert(
+            d, build_direction="LEFT_TO_RIGHT", table_attributes=table_attributes
+        )
         return (
-            """<button class="accordion">Final Test Summary</button><div class="panel"><p><pre>"""
-            + self.converter.convert(
-                self.results.tui_sections.short_test_summary.content, full=False
-            )
-            + "\n"
-            + self.converter.convert(
-                self.results.tui_sections.lastline.content, full=False
-            )
+            """<button class="accordion">Test Execution Info</button><div class="panel-open"><p><pre>"""
+            + dj
             + """</pre></p></div>"""
         )
 
     def create_environment_info(self, m, table_attributes) -> str:
         return (
-            """<button class="accordion">Environment</button><div class="panel"><p><pre>"""
+            """<button class="accordion">Environment</button><div class="panel-open"><p><pre>"""
             + json2table.convert(
                 m, build_direction="LEFT_TO_RIGHT", table_attributes=table_attributes
             )
             + """</pre></p></div>"""
         )
 
+    def get_js(self) -> str:
+        """
+        Gets JS from scripts.js and wraps them in <script> tags.
+        Note that in the scripts.js file, all scripts MUST e one-liners.
+        """
+        with open(Path(JS_FILE), "r") as f:
+            lines = f.readlines()
+        return "".join(f"<script>{line}</script>" for line in lines if line.strip("\n"))
+
     def create_trailer(self) -> str:
-        return """</script> </body> </html>"""
+        return f"""{self.get_js()} </script> </body> </html>"""
 
     def create_tabs(self) -> str:
         tabs_links = [
-            f"""<button class="tablinks" style="background-color: {TAB_METADATA_COLOR[section]}" id="defaultOpen" onclick="openTab(event, '{section}')" >{section}</button>"""
-            for section in TAB_METADATA
+            f"""<div class="sticky"> <span><button class="tablinks" style="background-color: {TAB_ABOUT_COLOR[section]}" id="defaultOpen" onclick="openTab(event, '{section}')" >{section}</button></span>"""
+            for section in TAB_ABOUT
         ]
 
         tabs_links.extend(
             [
-                f"""<button class="tablinks" style="background-color: {TABS_RESULTS_COLORS[section][0]}" onclick="openTab(event, '{section}')" >{section}</button>"""
+                f"""<span><button class="tablinks" style="background-color: {TABS_RESULTS_COLORS[section][0]}" onclick="openTab(event, '{section}')" >{section}</button></span>"""
                 for section in TABS_RESULTS
             ]
         )
 
+        # Dropdown for output sections
         tabs_links.extend(
             [
-                """<div class="dropdown"> <button class="dropbtn" style="color: white; background-color: gray">Output Sections</button> <div id="myDropdown" class="dropdown-content">"""
+                """<span class="dropdown"> <button class="dropbtn" style="color: #dddddd; background-color: gray">Output Sections</button> <span class="dropdown-content">"""
             ]
         )
-
         tabs_links.extend(
             [
-                f"""<button class="dropdown-item tablinks" style="background-color: {TABS_SECTIONS_COLORS[section]}" onclick="openTab(event, '{section}')" >{section}</button>"""
+                f"""<span><button class="dropdown-item tablinks" style="background-color: {TABS_SECTIONS_COLORS[section]}" onclick="openTab(event, '{section}')" >{section}</button></span>"""
                 for section in TABS_SECTIONS
             ]
         )
+        tabs_links.extend(["""</span> </span>"""])
 
-        tabs_links.extend(["""</div> </div>"""])
+        # Dropdown for fold-section actions
+        if self.results.tui_fold_level:
+            tabs_links.extend(
+                [
+                    """<span class="dropdown"> <button class="dropbtn" style="color: brown; background-color: #d9ead3">Fold Actions</button> <span class="dropdown-content">"""
+                ]
+            )
+            # tabs_links.extend(
+            #     [
+            #         f"""<span><button class="dropdown-item tablinks" style="background-color: {TABS_ACTIONS_COLORS[action]}" onclick="openAction(event, '{action}')" >{action}</button></span>"""
+            #         for action in TABS_ACTIONS
+            #     ]
+            # )
 
+            tabs_links.extend(
+                [
+                    """<span><button class="dropdown-item tablinks" style="background-color: #a8b3dc" onclick="toggleAllDetails()">Fold/Unfold Logs</button> </span>"""
+                ]
+            )
+            tabs_links.extend(
+                [
+                    """<span><button class="dropdown-item tablinks btn-rt" style="background-color: #b3dca8" id="toggle-details" onclick="toggleDetailsElements()">Show/Hide Fold Markers</button></span>"""
+                ]
+            )
+            tabs_links.extend(["""</span> </span>"""])
+
+        # Full Output tab
         tabs_links.extend(
             [
-                f"""<button class="tablinks" style="background-color: {TAB_FULL_OUTPUT_COLOR[section]}" onclick="openTab(event, '{section}')" >{section}</button>"""
+                f"""<span><button class="tablinks" style="background-color: {TAB_FULL_OUTPUT_COLOR[section]}" onclick="openTab(event, '{section}')" >{section}</button></span>"""
                 for section in TAB_FULL_OUTPUT
             ]
         )
 
-        tab_links_section = """<div class="tab">""" + "".join(tabs_links) + """</div>"""
+        if self.results.tui_fold_level:
+            tabs_links.extend(
+                [
+                    f"""<span><button class="tablinks" style="background-color: {TAB_FOLDED_OUTPUT_COLOR[section]}" onclick="openTab(event, '{section}')" >{section}</button></span> </div>"""
+                    for section in TAB_FOLDED_OUTPUT
+                ]
+            )
 
-        tab_links_section += """</div>"""
+        # tab_links_section += """</div>"""
+        # tabs_links.extend(
+        #     [
+        #         f"""<button class="dropdown-item tablinks" style="background-color: {TAB_ACTIONS_COLOR[action]}" onclick="toggleDetails()">Actions</button>"""
+        #         for action in TAB_ACTIONS
+        #     ]
+        # )
+        tab_links_section = (
+            """<span class="tab">""" + "".join(tabs_links) + """</span>"""
+        )
 
         tab_result_content = []
         for tab in TABS_RESULTS:
@@ -273,16 +361,22 @@ class HtmlPage:
         ]
         tab_sections = "".join(tab_section_content)
 
-        tab_metadata = f"""<div id="{TAB_METADATA[0]}" class="tabcontent"> <p>{self.get_metadata()}</p> </div>"""
+        tab_about = f"""<div id="{TAB_ABOUT[0]}" class="tabcontent"> <p>{self.get_metadata()}</p> </div>"""
 
         tab_full_output = f"""<div id="{TAB_FULL_OUTPUT[0]}" class="tabcontent"> <pre>{self.get_terminal_output()}</pre> </div>"""
 
+        if self.results.tui_fold_level:
+            tab_folded_output = f"""<span id="{TAB_FOLDED_OUTPUT[0]}" class="tabcontent"> <pre>{self.fold_terminal_output(self.results.tui_fold_level)}</pre> </span>"""
+        else:
+            tab_folded_output = ""
+
         return (
             tab_links_section
-            + tab_metadata
+            + tab_about
             + tab_results
             + tab_sections
             + tab_full_output
+            + tab_folded_output
         )
 
     def get_collapsible_results(self, outcome) -> str:
@@ -328,7 +422,7 @@ class HtmlPage:
 
         return (
             "<hr>"
-            + f"{self.create_testrun_summary()}<hr>"
+            + f"{self.create_final_test_summary()}<hr>"
             + f"{self.create_live_test_session_summary()}<hr>"
             + f"{self.create_test_execution_info()}<hr>"
             + f"{self.create_environment_info(m, table_attributes)}"
@@ -339,10 +433,77 @@ class HtmlPage:
             tout = str(f.read(), "utf-8")
         return self.converter.convert(tout, full=False)
 
-    def get_js(self) -> str:
-        with open(Path(JS_FILE), "r") as f:
-            lines = f.readlines()
-        return "".join(f"<script>{line}</script>" for line in lines if line.strip("\n"))
+    def get_terminal_output_ansi(self) -> str:
+        with open(TERMINAL_OUTPUT_FILE, "rb") as f:
+            tout = str(f.read(), "utf-8")
+        return tout
+
+    def get_line_level(self, line: str) -> str:
+        """Is this line a log entry (DEBUG, INFO, WARNING, ERROR, CRITICAL)?"""
+        for level, value in LOG_LEVEL_MAP.items():
+            if level in line:
+                return value
+
+    def fold_log_lines(self, lines: list, level: str) -> str:
+        """
+        Search each line of console output and look for a log level indicator (DEBUG, INFO, WARNING, ERROR, CRITICAL).
+        If a line contains a log level indicator, check if log level is <= the level passed to the function.
+        If the log level is <= the level passed to the function, then the line is folded.
+        If the log level is > than the level passed to the function, then the line is not folded.
+        """
+        html_lines = []
+        html_str = ""
+        fold_started = False
+        for l in lines:
+            line = self.converter.convert(l, full=False)
+            this_line_log_level = self.get_line_level(l)
+            if this_line_log_level:
+                if this_line_log_level <= LOG_LEVEL_MAP[level]:
+                    if not fold_started:
+                        html_str += (
+                            f"<details><summary style='nobr'>Folded {level}</summary>"
+                        )
+                        fold_started = True
+                else:
+                    if fold_started:
+                        html_str += "</details>"
+                        fold_started = False
+                html_str += line + "\n"
+            else:
+                if fold_started:
+                    html_str += "</details>"
+                    fold_started = False
+                    html_str += line + "\n"
+                else:
+                    html_str += line + "\n"
+        return html_str
+
+    def fold_terminal_output(self, level: str) -> str:
+        terminal_output_ansi = self.get_terminal_output_ansi()
+        lines = terminal_output_ansi.splitlines()
+        return self.fold_log_lines(lines, level)
+
+    # def fold_tracebacks(self, lines):
+    #     html_lines = []
+    #     folding = False
+    #     for line in lines:
+    #         if '_ test_' in line:
+    #             if not folding:
+    #                 html_lines.append("<details><summary>Folded DEBUG or INFO</summary>")
+    #                 folding = True
+    #         elif 'Captured log' in line and '-' in line:
+    #             if folding:
+    #                 html_lines.append('</details>')
+    #                 folding = False
+    #         html_lines.append(self.converter.convert(line, full=False))
+    #     if folding:
+    #         html_lines.append('</details>')
+    #     return '\n'.join(html_lines)
+
+    # def fold_terminal_output(self) -> str:
+    #     terminal_output_ansi = self.get_terminal_output_ansi()
+    #     lines = terminal_output_ansi.splitlines()
+    #     return self.fold_tracebacks(lines)
 
 
 def main():
@@ -351,16 +512,20 @@ def main():
     page.remove_tabs_without_content()
     html_header = page.create_header()
     html_tabs = page.create_tabs()
+    # html_action_button = """<button class="dropdown-item tablinks btn-rt" style="background-color: rgba(249, 123, 64, 0.95)" onclick="toggleDetails()">Actions</button>"""
     html_trailer = page.create_trailer()
-    my_js = page.get_js()
-    html_out = html_header + html_tabs + "\n" + my_js + html_trailer
+    html_out = html_header + html_tabs + html_trailer
 
-    if "tui_htmlfile" in results.tui_test_info and results.tui_test_info['tui_htmlfile']:
-        HTML_OUTPUT_FILE = Path(PYTEST_TUI_FILES_DIR / results.tui_test_info['tui_htmlfile'])
+    global HTML_OUTPUT_FILE
+    if (
+        "tui_htmlfile" in results.tui_test_info
+        and results.tui_test_info["tui_htmlfile"].name
+    ):
+        HTML_OUTPUT_FILE = Path(
+            PYTEST_TUI_FILES_DIR / results.tui_test_info["tui_htmlfile"]
+        )
     with open(HTML_OUTPUT_FILE, "w+") as f:
         f.write(html_out)
-
-    # Open in browser
     webbrowser.open(f"file://{HTML_OUTPUT_FILE._str}")
 
     # Open in browser if autolaunch_html config is set
