@@ -2,8 +2,7 @@ import itertools
 import pickle
 import re
 import tempfile
-
-# from concurrent.futures.thread import ThreadPoolExecutor
+from concurrent.futures.thread import ThreadPoolExecutor
 from datetime import datetime, timezone
 from io import StringIO
 from types import SimpleNamespace
@@ -17,7 +16,7 @@ from _pytest.reports import TestReport
 from strip_ansi import strip_ansi
 
 # from pytest_tui.__main__ import tui_launch
-# from pytest_tui.html import main as tuihtml
+from pytest_tui.html_gen import main as tuihtml
 from pytest_tui.utils import (
     HTML_OUTPUT_FILE,
     TERMINAL_OUTPUT_FILE,
@@ -51,11 +50,26 @@ def pytest_addoption(parser) -> None:
     group.addoption(
         "--tui",
         action="store_true",
-        help="Enable the pytest-tui plugin. Both text user interface (TUI) and HTML output are supported.\nRun TUI with console command 'tui'; run HTML report with 'tuih'.",
+        help=(
+            "Enable the pytest-tui plugin. Both text user interface (TUI) and HTML"
+            " output are supported.\nRun TUI with console command 'tui'; run HTML"
+            " report with 'tuih'."
+        ),
     )
     group.addoption(
         "--tui-htmlfile",
-        help="Specify a non-default name for the HTML report file. Default is 'html-report.html,' and will be placed in the ptt_files/ folder.",
+        help=(
+            "Specify a non-default name for the HTML report file. Default is"
+            " 'html-report.html,' and will be placed in the ptt_files/ folder."
+        ),
+    )
+    group.addoption(
+        "--tui-fold-level",
+        help=(
+            "Enable auto-folding of log messages in the HTML report, and specify the"
+            " level at which folding occurs. When enabled, log messages at or below the"
+            " specified level will be folded."
+        ),
     )
 
 
@@ -143,6 +157,21 @@ def pytest_cmdline_main(config: Config) -> None:
     else:
         config._tui_htmlfile = config.option.tui_htmlfile
 
+    # Set default fold level, if specified on the command line
+    if (
+        hasattr(config.option, "tui_fold_level")
+        and not config.option.tui_fold_level
+        or not hasattr(config.option, "tui_fold_level")
+    ):
+        config._tui_fold_level = None
+    else:
+        config._tui_fold_level = (
+            config.option.tui_fold_level
+            if config.option.tui_fold_level.lower()
+            in ["debug", "info", "warning", "error", "critical"]
+            else None
+        )
+
 
 def pytest_report_teststatus(report: TestReport, config: Config) -> None:
     # Don't process any TUI-specific code if the plugin is not enabled
@@ -212,16 +241,14 @@ def pytest_configure(config: Config) -> None:
         return
 
     # Examine Pytest terminal output to mark different sections of the output.
-    # This code is based on the code in pytest's `pastebin.py`.
+    # This code is based on code in pytest's 'pastebin.py'.
     tr = config.pluginmanager.getplugin("terminalreporter")
     if tr is not None:
-
         # Save the old terminal writer instance so we can restore it later
         oldwrite = tr._tw.write
 
         # identify and mark each results section
         def tee_write(s, **kwargs):
-
             # Check to see if current line is a section start marker
             if re.search(test_session_starts_matcher, s):
                 config._tui_current_section = "test_session_starts"
@@ -269,7 +296,6 @@ def pytest_configure(config: Config) -> None:
             if config._tui_current_section == "short_test_summary" and re.search(
                 short_test_summary_test_matcher, strip_ansi(s)
             ):
-
                 outcome = re.search(
                     short_test_summary_test_matcher, strip_ansi(s)
                 ).groups()[0]
@@ -324,7 +350,7 @@ def populate_rerun_groups(config: Config) -> List[TuiRerunTestGroup]:
         if test_result.outcome != "RERUN":
             for group in rerun_test_groups:
                 if group.fqtn == test_result.fqtn:
-                    group.final_outcome = test_result.outcome
+                    group.test_outcome = test_result.outcome
                     group.final_test = test_result
     for group in rerun_test_groups:
         group.full_test_list = group.forerunners + [group.final_test]
@@ -381,10 +407,11 @@ def pytest_unconfigure(config: Config) -> None:
                 "session_start_time": config._tui_session_start_time,
                 "session_end_time": config._tui_session_end_time,
                 "session_duration": config._tui_session_duration,
-                "tui_rerun_test_groups": config._tui_rerun_test_groups,
                 "tui_test_results": config._tui_test_results,
+                "tui_rerun_test_groups": config._tui_rerun_test_groups,
                 "tui_sections": config._tui_sections,
                 "tui_htmlfile": config._tui_htmlfile,
+                "tui_fold_level": config._tui_fold_level,
             },
             file,
         )
@@ -402,7 +429,7 @@ def pytui_launch(config: Config) -> None:
         capmanager.suspend_global_capture(in_=True)
     finally:
         # re-enable capturing
-        # with ThreadPoolExecutor() as executor:
-        #     executor.submit(tuihtml)
+        with ThreadPoolExecutor() as executor:
+            executor.submit(tuihtml)
         # tui_launch()
         capmanager.resume_global_capture()
